%{
  open Expr
%}

%token EOF

%token <string> VAR
%token <string> STRING
%token <int> INT

%token <string> BIN_MULT
%token <string> BIN_PLUS
%token <string> BIN_CMP
%token NOT
%token AND
%token OR

%token LPAR RPAR

%token IF THEN ELSE
%token WHILE DO
%token ASSIGN
%token SKIP
%token SEQ
%token BEGIN END

/* Les non-terminaux par lesquels l'analyse peut commencer,
 * et la donnée de leurs types. */

%start terminated_expr
%type <Expr.t> terminated_expr

%%

terminated_expr:
  | cmd EOF { $1 }
  | expr EOF { $1 }

expr:
  | INT                            { Int $1 }
  | VAR                            { Var $1 }
  | STRING                         { String $1 }
  | LPAR expr RPAR                 { $2 }
  | expr BIN_PLUS expr             { App ($2,[$1;$3]) }
  | NOT expr                       { App ("not",[$2]) }

cmd:
  | VAR ASSIGN expr           { Assign($1,$3) }
  | SKIP                      { Skip }
  | cmd SEQ cmd               { Seq($1,$3) }
  | IF expr THEN cmd ELSE cmd { Ite($2,$4,$6) }
  | WHILE expr DO cmd         { While($2,$4) }
  | BEGIN cmd END             { $2 }